@@ -1,9 +1,9 @@
 import inspect
 import sys
 
-from weaveio.hierarchy import Optional, Multiple, Hierarchy
-from weaveio.opr3.hierarchy import Spectrum, Single, FibreTarget, Stack, \
-    OBStack, OB, ArmConfig, Superstack, OBSpec, Supertarget, WeaveTarget, RawSpectrum, _predicate, Spectrum1D, MeanFlux, WavelengthHolder
+from weaveio.hierarchy import Optional, Multiple, Hierarchy, OneOf
+from weaveio.opr3.hierarchy import Spectrum, Single, FibreTarget, Stacked, \
+    Stack, OB, ArmConfig, Superstack, OBSpec, Supertarget, WeaveTarget, RawSpectrum, _predicate, Spectrum1D, MeanFlux
 
 
 class L1(Hierarchy):
@@ -13,7 +13,6 @@ class L1(Hierarchy):
 class L1Spectrum(Spectrum1D, L1):
     is_template = True
     children = [Optional('self', idname='adjunct')]
-    parents = [WavelengthHolder]
     products = ['flux', 'ivar', 'sensfunc']
     factors = Spectrum.factors + ['nspec', 'snr'] + MeanFlux.as_factors('g', 'r', 'i', 'gg', 'bp', 'rp')
 
@@ -22,8 +21,9 @@ class NoSS(Spectrum1D):
     plural_name = 'nosses'
     singular_name = 'noss'
     products = ['flux', 'ivar']
-    parents = [L1Spectrum]
+    parents = [OneOf(L1Spectrum, one2one=True)]
     children = [Optional('self', idname='adjunct')]
+    identifier_builder = ['l1_spectrum']
 
 
 class L1SingleSpectrum(L1Spectrum, Single):
@@ -32,8 +32,8 @@ class L1SingleSpectrum(L1Spectrum, Single):
     """
     singular_name = 'l1single_spectrum'
     plural_name = 'l1single_spectra'
-    parents = L1Spectrum.parents + [RawSpectrum, FibreTarget]
-    version_on = ['raw_spectrum', 'fibre_target']
+    parents = [FibreTarget, RawSpectrum, ArmConfig] + L1Spectrum.parents  # order matters for speed! filters by fibretarget, then rawspectrum, then armconfig
+    identifier_builder = ['raw_spectrum', 'fibre_target', 'arm_config']
     factors = L1Spectrum.factors + [
         'rms_arc1', 'rms_arc2', 'resol', 'helio_cor',
         'wave_cor1', 'wave_corrms1', 'wave_cor2', 'wave_corrms2',
@@ -41,40 +41,44 @@ class L1SingleSpectrum(L1Spectrum, Single):
         'sky_shift', 'sky_scale']
 
 
-class L1StackSpectrum(L1Spectrum, Stack):
+class L1StackedSpectrum(L1Spectrum, Stacked):
     is_template = True
     singular_name = 'l1stack_spectrum'
     plural_name = 'l1stack_spectra'
+    parents = [Multiple(L1SingleSpectrum, 2)] + L1Spectrum.parents
+    identifier_builder = ['l1single_spectra']
 
 
-class L1OBStackSpectrum(L1StackSpectrum, OBStack):
+class L1StackSpectrum(L1StackedSpectrum, Stack):
     """
     A stacked spectrum row processed from > 1 single spectrum, belonging to one fibretarget but many runs within the same OB.
     """
-    singular_name = 'l1obstack_spectrum'
-    plural_name = 'l1obstack_spectra'
-    parents = L1StackSpectrum.parents + [Multiple(L1SingleSpectrum, 2, constrain=(OB, FibreTarget, ArmConfig))]
-    version_on = ['l1single_spectra']
+    singular_name = 'l1stack_spectrum'
+    plural_name = 'l1stack_spectra'
+    parents = [Multiple(L1SingleSpectrum, 2, constrain=(FibreTarget, OB, ArmConfig))] + L1Spectrum.parents
 
 
-class L1SuperstackSpectrum(L1StackSpectrum, Superstack):
+class L1SuperstackSpectrum(L1StackedSpectrum, Superstack):
     """
     A stacked spectrum row processed from > 1 single spectrum, belonging to one fibretarget but many runs within the same OBSpec.
     """
     singular_name = 'l1superstack_spectrum'
     plural_name = 'l1superstack_spectra'
-    parents = L1StackSpectrum.parents + [Multiple(L1SingleSpectrum, 2, constrain=(OBSpec, FibreTarget, ArmConfig))]
-    # version_on = ['l1single_spectra', 'fibre_target']
+    parents = [Multiple(L1SingleSpectrum, 2, constrain=(FibreTarget, OBSpec, ArmConfig))] + L1Spectrum.parents
 
 
-class L1SupertargetSpectrum(L1Spectrum, Supertarget):
+class L1SupertargetSpectrum(L1StackedSpectrum, Supertarget):
     """
     A stacked spectrum row processed from > 1 single spectrum, belonging to one weavetarget over many different OBSpecs.
+    A weavetarget can have more than supertarget spectrum so an ID is added to the filename
+    todo: This structure needs looking at.
     """
     singular_name = 'l1supertarget_spectrum'
     plural_name = 'l1supertarget_spectra'
-    parents = L1Spectrum.parents + [Multiple(L1SingleSpectrum, 2, constrain=(WeaveTarget, ArmConfig))]
-    # version_on = ['l1single_spectra', 'weave_target']
+    factors = ['id']
+    parents = [Multiple(L1SingleSpectrum, 2, constrain=(WeaveTarget, ArmConfig))] + L1Spectrum.parents
+    identifier_builder = ['l1single_spectra', 'id']
+
 
 
 hierarchies = [i[-1] for i in inspect.getmembers(sys.modules[__name__], _predicate)]
\ No newline at end of file
